#include <iostream>
#include <vector>

const int MAX_N = 100 + 2;
std::vector<int> graph[MAX_N];
int pipe[MAX_N][MAX_N];
int n;
bool used[MAX_N];

int aug_path(int curr_node, const int sink, int curr_flow) {
	if(curr_node == sink) {
		return curr_flow;
	}

	for(auto nei : graph[curr_node]) {
		if(used[nei]) continue;
		used[nei] = true;
		const int minf = std::min(curr_flow, pipe[curr_node][nei]);
		int f = aug_path(nei, sink, minf);
		used[nei] = false;
		if(f > 0) {
			pipe[curr_node][nei] -= f;
			pipe[nei][curr_node] += f;
			return f;
		}
	}

	return 0;
}

int do_maxflow(const int source, const int sink) {
	std::fill(used, used+n, false);
	int tot = 0;
	while(true) {
		int f = aug_path(source, sink, 1<<30);
		tot += f;
		if(f == 0) break;
	}
	return tot;
}

int main() {
	std::cin.tie(nullptr);
	std::ios::sync_with_stdio(false);

	int m;
	std::cin >> n >> m;
	for(int i = 0;i < m;i ++) {
		int a, b;
		std::cin >> a >> b;
		a --;
		b --;
		graph[a].push_back(b);
		graph[b].push_back(a);
		pipe[a][b] ++;
		pipe[b][a] ++;
	}

	int q;
	std::cin >> q;

	const int source = 0;
	const int sink = n-1;

	int tot = 0;
	tot += do_maxflow(source, sink);

	std::cout << tot << '\n';

	for(int i = 0;i < q;i ++){
		int a, b;
		std::cin >> a >> b;
		a --;
		b --;
		graph[a].push_back(b);
		graph[b].push_back(a);
		pipe[a][b] ++;
		pipe[b][a] ++;

		tot += do_maxflow(source, sink);
		std::cout << tot << std::endl;
	}

	return 0;
}
