#include <iostream>
#include <cstdint>
#include <random>
#include <utility>
#include <cassert>

const uint32_t MAX_N = 1e5 + 12;

template <class T>
std::ostream& operator<< (std::ostream &s, const std::vector<T> &v) {
	for (const T &i : v) {
		s << i << " ";
	}
	return s;
}

class Node {
private:
	static std::mt19937 rnd;

	Node *par;
	Node *l, *r;
	uint32_t subTree;
	const uint32_t p;

	void reeval() {
		subTree = 1;
		subTree += l ? l->subTree : 0;
		subTree += r ? r->subTree : 0;
	}

	void set_l(Node *n) {
		l = n;
		if (n) {
			n->par = this;
		}
		reeval();
	}
	void set_r(Node *n) {
		r = n;
		if (n) {
			n->par = this;
		}
		reeval();
	}

	void detach_l() {
		if (l) {
			l->par = nullptr;
		}
	}
	void detach_r() {
		if (r) {
			r->par = nullptr;
		}
	}

public:
	uint32_t val;
	Node(): par(nullptr), l(nullptr), r(nullptr), subTree(1), p(rnd()) {}

	static std::pair<Node*, Node*> split(Node *n, uint32_t lCnt) {
		if (n == nullptr) {
			return {nullptr, nullptr};
		}
		uint32_t lSubTree = n->l ? n->l->subTree : 0;
		if (lCnt <= lSubTree) {
			n->detach_l();
			auto t = split(n->l, lCnt);
			n->set_l(t.second);
			return {t.first, n};
		} else {
			n->detach_r();
			auto t = split(n->r, lCnt - lSubTree - 1);
			n->set_r(t.first);
			return {n, t.second};
		}
	}
	static Node* merge (Node *l, Node *r) {
		if ((l == nullptr) || (r == nullptr)) {
			return r ? r : l;
		}

		if (l->p > r->p) {
			l->detach_r();
			l->set_r(merge(l->r, r));
			return l;
		} else {
			r->detach_l();
			r->set_l(merge(l, r->l));
			return r;
		}
	}
	uint32_t ind() const {
		uint32_t ans = 0;
		if (l) {
			ans += l->subTree;
		}
		const Node *v = par;
		const Node *prev = this;
		while (v != nullptr) {
			if (v->r == prev) {
				ans++;
				if (v->l) {
					ans += v->l->subTree;
				}
			}
			prev = v;
			v = v->par;
		}
		return ans;
	}

	void print() const {
		if (l) {
			l->print();
		}
		std::cerr << val << " ";
		if (r) {
			r->print();
		}
	}

	bool sorted() const {
		if (val != ind()) {
			return false;
		} else if ((l) && (!l->sorted())) {
			return false;
		} else if ((r) && (!r->sorted())) {
			return false;
		} else {
			return true;
		}
	}
};

std::mt19937 Node::rnd(27071999);

std::vector<uint32_t> p;
uint32_t n, k;

void input() {
	std::ios::sync_with_stdio(false);
	std::cin.tie(nullptr);

	std::cin >> n >> k;
	std::vector<uint32_t> p1;
	std::vector<uint32_t> p2;

	p1.resize(n);
	p2.resize(n);
	for (uint32_t &i : p1) {
		std::cin >> i;
		i--;
	}

	for (uint32_t i = 0;i < n;i++) {
		uint32_t a;
		std::cin >> a;
		p2[a - 1] = i;
	}

	for (uint32_t i : p1) {
		p.push_back(p2[i]);
	}
}

Node *root = nullptr;
Node nodes[MAX_N];
void build_treap() {
	for (uint32_t i = 0;i < n;i++) {
		nodes[i].val = i;
	}
	for (uint32_t i = 0;i < n;i++) {
		root = Node::merge(root, &nodes[p[i]]);
	}

//	root->print();
//	std::cerr << "\n";
//	for (uint32_t i = 0;i < n;i++) {
//		std::cerr << nodes[i].ind() << " ";
//	}
//	std::cerr << "\n";
}

bool crude_check() {
	for (uint32_t i = 0;i < p.size();i++) {
		if (p[i] % k != i % k) {
			return false;
		}
	}
	return true;
}

void place_val(uint32_t i) {
	assert(i + 2 * k < n);
	uint32_t cInd = nodes[i].ind();

//	std::cerr << i << " at " << cInd << " :\n";

	if (cInd + k >= n) {
		auto s1 = Node::split(root, n - 2 * k);
		auto s2 = Node::split(s1.second, k);
		root = Node::merge(s1.first, Node::merge(s2.second, s2.first));

//		std::cerr << "extra swap: "; 
//		root->print();
//		std::cerr << std::endl;

		cInd = nodes[i].ind();
	}
	auto s1 = Node::split(root, i);
	auto s2 = Node::split(s1.second, cInd - i);
	auto s3 = Node::split(s2.second, k);
	root = Node::merge(s1.first, s3.first);
	root = Node::merge(root, s2.first);
	root = Node::merge(root, s3.second);

//	std::cerr << "final: ";
//	root->print();
//	std::cerr << "\n";
}

int main() {
	input();
	if (! crude_check()) {
		std::cout << "0\n";
		return 0;
	}
	build_treap();

	for (uint32_t i = 0;i + 2 * k < n;i++) {
		place_val(i);
	}

	bool possib = false;
	if (root->sorted()) {
		possib = true;
	}
	auto s1 = Node::split(root, n - 2 * k);
	auto s2 = Node::split(s1.second, k);
	root = Node::merge(s1.first, Node::merge(s2.second, s2.first));
	if (root->sorted()) {
		possib = true;
	}
	std::cout << possib << "\n";
}
