//Dim4ou
#include <cstdint>
#include <iostream>
#include <algorithm>

const uint32_t MAX_E = 1.1e5;
const uint32_t MAX_V = 101;

struct Vertex;

struct Edge {
	Vertex *to;
	uint32_t cap;
	Edge *rev;

	Edge *next;
	Edge *prev;
	
	static void add_edge(Edge* e, Edge *next) {
		e->prev = nullptr;
		e->next = next;
		if (next) {
			next->prev = e;
		}
	}
	static void rem_edge(Edge *e) {
		if (e->prev) {
			e->prev->next = e->next;
		}
		if (e->next) {
			e->next->prev = e->prev;
		}
	}

	void push_flow();
};

Edge edgeMem[MAX_E];
Edge *edgePtr = edgeMem;

uint32_t vCnt;
uint32_t qCnt;

struct Vertex {
	Edge* edges[MAX_V];
	Edge* eIn;
	bool vis;
	uint32_t ind;

	Vertex(): eIn(nullptr), vis(false) {
		std::fill(edges, edges + MAX_V, nullptr);
	}

	void add_edge(Vertex *to) {
		Edge *e1 = edgePtr++;
		Edge *e2 = edgePtr++;

		e1->to = to;
		e2->to = this;
		e1->cap = e2->cap = 1;

		e1->rev = e2;
		e2->rev = e1;

		Edge::add_edge(e1, edges[to->ind]);
		Edge::add_edge(e2, to->edges[ind]);

		edges[to->ind] = e1;
		to->edges[ind] = e2;
	}
	bool dfs();
	void push_flow();
};


Vertex graph[MAX_V];
Vertex *source, *sink;

void Edge::push_flow() {
	cap--;
	rev->cap++;
	if (cap == 0) {
		Vertex *v = rev->to;
		uint32_t ind = to->ind;

		Edge::rem_edge(this);
		if (v->edges[ind] == this) {
			v->edges[ind] = next;
		}
	}

	if (rev->cap == 1) {
		Vertex *v = to;
		uint32_t ind = rev->to->ind;
		Edge::add_edge(rev, v->edges[ind]);
		v->edges[ind] = rev;
	}
}

void input() {
	std::ios::sync_with_stdio(false);
	std::cin.tie(nullptr);

	uint32_t eCnt;
	std::cin >> vCnt >> eCnt;
	for (uint32_t i = 0;i < vCnt;i++) {
		graph[i].ind = i;
	}
	source = &graph[0];
	sink = &graph[vCnt - 1];

	for (uint32_t i = 0;i < eCnt;i++) {
		uint32_t a, b;
		std::cin >> a >> b;
		graph[a - 1].add_edge(&graph[b - 1]);
	}
}

void clear_vis() {
	for (uint32_t i = 0;i < vCnt;i++) {
		graph[i].vis = false;
		graph[i].eIn = nullptr;
	}
}

bool Vertex::dfs() {
	if (this == sink) {
		return true;
	} else if (vis) {
		return false;
	}
	vis = true;
	//std::cerr << "Vis " << ind << "\n";
	for (uint32_t i = 0;i < vCnt;i++) {
		if (edges[i] == nullptr) {
			continue;
		}
		if (!graph[i].vis) {
			graph[i].eIn = edges[i];
			//std::cerr << "Edge  " << ind << " -> " << graph[i].ind << "\n";
			if (graph[i].dfs()) {
				return true;
			}
		}
	}
	return false;
}

uint32_t flow = 0;

void Vertex::push_flow() {
	if (eIn == nullptr) {
		flow++;
		return;
	} else {
		eIn->push_flow();
		eIn->rev->to->push_flow();
	}
}


void init_flow() {
	while (true) {
		clear_vis();
		if (source->dfs()) {
			sink->push_flow();
		} else {
			break;
		}
	}
}

int main() {
	input();
	init_flow();
	std::cin >> qCnt;
	std::cout << flow << "\n";

	for (uint32_t i = 0;i < qCnt;i++) {
		uint32_t a, b;
		std::cin >> a >> b;
		a--;
		b--;
		graph[a].add_edge(&graph[b]);

		if ((graph[b].vis) && (!graph[a].vis)) {
			std::swap(a, b);
		}
		if ((graph[a].vis) && (!graph[b].vis)) {
			graph[b].eIn = graph[a].edges[b];
			if (graph[b].dfs()) {
				sink->push_flow();
				clear_vis();
				source->dfs();
			}
		}
//		std::cerr << "visited :";
//		for (uint32_t j = 0;j < vCnt;j++) {
//			std::cerr << graph[j].vis << " ";
//		}
//		std::cerr << "\n";


		std::cout << flow << "\n";
	}
}
