#pragma optimize (-Ofast)
#include <bits/stdc++.h>
#define endl '\n'
using namespace std;

const int32_t MAX_N = 100;
const int32_t INF = 2e9;

class Graph {
private:
	class Edge;
	class Node {
	public:
		bool isVisited;
		int32_t dep;
		vector< Edge > v;	
	
		Node() {
			isVisited = false;
			v.clear();
		}
	};

	class Edge {
	public:
		int32_t rev, flow, cap;
		Node *to;

		Edge(Node *to = nullptr, int32_t flow = 0, int32_t cap = 0, int32_t rev = 0) 
			: to(to), flow(flow), cap(cap), rev(rev) { }
	};

public:
	int32_t cntNodes, maxFlow;
	Node nodes[MAX_N + 5];

	void Init(int32_t n) {
		cntNodes = n;
	}

	void AddEdge(int32_t x, int32_t y) {
		int32_t revY = (int32_t) nodes[x].v.size();
		int32_t revX = (int32_t) nodes[y].v.size();

		nodes[x].v.push_back(Edge(&nodes[y], 0, 1, revX));
		nodes[y].v.push_back(Edge(&nodes[x], 1, 1, revY));
	}

	void Bfs() {
		queue< pair< Node*, int32_t > > q;

		q.push({ &nodes[cntNodes], 0 });
		nodes[cntNodes].isVisited = true;
		nodes[cntNodes].dep = 0;
		
		while(!q.empty()) {
			Node *nd = q.front().first;
			int32_t dep = q.front().second;
			q.pop();
			
			if(nd == &nodes[1]) {
				return;
			}

			for(auto &e : nd->v) {
				if(!e.to->isVisited && e.to->v[e.rev].flow != e.to->v[e.rev].cap) {
					e.to->isVisited = true;
					e.to->dep = dep + 1;
					
					q.push({ e.to, dep + 1 });
				}
			}
		}
	}

	int32_t Dfs(Node *nd, int32_t minLeftCap) {
		if(nd == &nodes[cntNodes]) {
			maxFlow += minLeftCap;
			return minLeftCap;
		}
		
		for(auto &e : nd->v) {
			if(!e.to->isVisited && e.flow != e.cap && e.to->dep == nd->dep - 1) {
				int32_t newMinLeftCap = min(minLeftCap, e.cap - e.flow);

				if(newMinLeftCap == 0) {
					continue;
				}

				newMinLeftCap = Dfs(e.to, newMinLeftCap);

				e.flow += newMinLeftCap;
				e.to->v[e.rev].flow -= newMinLeftCap;

				return newMinLeftCap;
			}
		}
		
		nd->isVisited = true;
		return 0;
	}

	int32_t CalcMaxFlow() {
		while(1) {
			for(int32_t i = 1; i <= cntNodes; i++) {
				nodes[i].dep = -1;
				nodes[i].isVisited = false;
			}
			
			Bfs();
			
			if(!nodes[1].isVisited) {
				break;
			}
			
			for(int32_t i = 1; i <= cntNodes; i++) {
				nodes[i].isVisited = false;
			}

			while(1) {
				int32_t lastFlow = maxFlow;
				
				Dfs(&nodes[1], INF);
				
				if(lastFlow == maxFlow) {
					break;
				}
			}
		}

		return maxFlow;
	}
};

Graph g;
	
int main() {
	ios_base::sync_with_stdio(false);
	cin.tie(nullptr);

	int32_t n, m;
	cin >> n >> m;
	
	g.Init(n);
	for(int32_t i = 0; i < m; i++) {
		int32_t x, y;
		cin >> x >> y;

		g.AddEdge(x, y);
		g.AddEdge(y, x);
	}

	cout << g.CalcMaxFlow() << endl;

	int32_t q;
	cin >> q;
	for(int32_t i = 0; i < q; i++) {
		int32_t x, y;
		cin >> x >> y;
		
		g.AddEdge(x, y);
		g.AddEdge(y, x);

		cout << g.CalcMaxFlow() << endl;
	}
}
