#include <bits/stdc++.h>
#define endl '\n'
using namespace std;

const uint16_t MAX_N = 100;
const uint16_t INF = 1e4;

class Graph {
private:
	class Edge;
	class Node {
	public:
		bool isVisited;
		uint16_t dep;
		vector< Edge > v;	
	
		Node() {
			isVisited = false;
			v.clear();
		}
	};

	class Edge {
	public:
		uint16_t rev, flow, cap;
		Node *to;

		Edge(Node *to = nullptr, uint16_t flow = 0, uint16_t cap = 0, uint16_t rev = 0) 
			: to(to), flow(flow), cap(cap), rev(rev) { }
	};

public:
	uint16_t cntNodes, maxFlow;
	Node nodes[MAX_N + 1];

	void Init(uint16_t n) {
		cntNodes = n;
	}

	void AddEdge(uint16_t x, uint16_t y) {
		uint16_t revY = (uint16_t) nodes[x].v.size();
		uint16_t revX = (uint16_t) nodes[y].v.size();

		nodes[x].v.push_back(Edge(&nodes[y], 0, 1, revX));
		nodes[y].v.push_back(Edge(&nodes[x], 0, 1, revY));
	}

	void Bfs() {
		queue< pair< Node*, uint16_t > > q;

		q.push({ &nodes[cntNodes], 0 });
		nodes[cntNodes].isVisited = true;
		nodes[cntNodes].dep = 0;
		
		while(!q.empty()) {
			Node *nd = q.front().first;
			uint16_t dep = q.front().second;
			q.pop();
			
			if(nd == &nodes[1]) {
				return;
			}

			for(auto &e : nd->v) {
				if(!e.to->isVisited && e.to->v[e.rev].flow != e.to->v[e.rev].cap) {
					e.to->isVisited = true;
					e.to->dep = dep + 1;
					
					q.push({ e.to, dep + 1 });
				}
			}
		}
	}

	uint16_t Dfs(Node *nd, uint16_t minLeftCap) {
		if(nd == &nodes[cntNodes]) {
			maxFlow += minLeftCap;
			return minLeftCap;
		}
		
		for(auto &e : nd->v) {
			if(!e.to->isVisited && e.flow != e.cap && e.to->dep == nd->dep - 1) {
				uint16_t newMinLeftCap = min(minLeftCap, (uint16_t) (e.cap - e.flow));

				if(newMinLeftCap == 0) {
					continue;
				}

				newMinLeftCap = Dfs(e.to, newMinLeftCap);

				e.flow += newMinLeftCap;
				e.to->v[e.rev].flow -= newMinLeftCap;

				return newMinLeftCap;
			}
		}
		
		nd->isVisited = true;
		return 0;
	}

	uint16_t CalcMaxFlow() {
		while(1) {
			for(uint16_t i = 1; i <= cntNodes; i++) {
				nodes[i].dep = -1;
				nodes[i].isVisited = false;
			}
			
			Bfs();
			
			if(!nodes[1].isVisited) {
				break;
			}
			
			for(uint16_t i = 1; i <= cntNodes; i++) {
				nodes[i].isVisited = false;
			}

			while(1) {
				uint16_t lastFlow = maxFlow;
				
				Dfs(&nodes[1], INF);
				
				if(lastFlow == maxFlow) {
					break;
				}
			}
		}

		return maxFlow;
	}
};

Graph g;
	
int main() {
	ios_base::sync_with_stdio(false);
	cin.tie(nullptr);

	uint16_t n, m;
	//cin >> n >> m;
	scanf(" %d %d", &n, &m);

	g.Init(n);
	for(uint16_t i = 0; i < m; i++) {
		uint16_t x, y;
		//cin >> x >> y;
		scanf(" %d %d", &x, &y);

		g.AddEdge(x, y);
		//g.AddEdge(y, x);
	}

	//cout << g.CalcMaxFlow() << endl;
	printf("%d\n", g.CalcMaxFlow());

	uint16_t q;
	//cin >> q;
	scanf(" %d", &q);
	for(uint16_t i = 0; i < q; i++) {
		uint16_t x, y;
		//cin >> x >> y;
		scanf(" %d %d", &x, &y);

		g.AddEdge(x, y);
		//g.AddEdge(y, x);

		//cout << g.CalcMaxFlow() << endl;
		printf("%d\n", g.CalcMaxFlow());
	}
}
